require "cat_engine/engine"
require "nokogiri"

module CatEngine
  class Bank < Struct.new(:id, :history, :full_history)
    def paired?
      id.include? "Illness Impact"
    end

    def name
      xml.root["Name"]
    end

    def current_question
      Question.new(xml: xml) unless finished?
    end

    def valid_answer?(response_id)
      engine.valid_history?(id, [history, response_id].join(" "))
    end

    def finished?
      xml.root.name == "Results"
    end

    def theta
      return unless xml.root.key?("Theta")
      (10 * xml.root["Theta"].to_f + 50).round
    end

    def standard_error
      return unless xml.root.key?("StdError")
      (10 * xml.root["StdError"].to_f).round
    end

    def to_h
      {
        finished: finished?,
        theta: theta,
        standard_error: standard_error,
      }
    end

    alias_method :to_param, :id

    private

    def xml
      engine.process(id, history)
    end

    def engine
      id.include?(".flat") ? FlatEngine.new(full_history) : Engine.new
    end
  end
end

