require "nokogiri"
require "memoist"
require "cat_engine/config"

module CatEngine
  class Engine
    extend Memoist
    class EngineError < StandardError; end

    def self.compile
      gmcs = [`which gmcs`.chomp, `which mcs`.chomp].find { |path| path != "" }
      system "cd #{CatEngine.cat_engine_source_path} && #{gmcs} -debug -out:#{CatEngine.cat_engine_executable_path} *.cs"
    end

    def process id, history
      command = %(cd #{CatEngine.cat_path} && mono --debug #{CatEngine.cat_engine_executable_path} "#{id}" #{history} 2>&1)
      output = `#{command}`.chomp
      if $?.success?
        Nokogiri::XML.parse(output, nil, "utf-8")
      else
        raise EngineError.new(output)
      end
    end
    memoize :process

    def valid_history? id, history
      process id, history
      true
    rescue EngineError => e
      false
    end
  end
end

