require "nokogiri"
require "active_model"

module CatEngine
  class Question < Struct.new(:id, :promis_id, :context, :title, :response_before, :responses)
    include ActiveModel::Conversion
    extend ActiveModel::Naming

    def initialize attributes = nil
      assign_attributes(attributes || {})
    end

    def context_and_title
      [context, title].select(&:present?).join(" ")
    end

    def xml= xml
      assign_attributes XMLMapper.new(xml).to_h
    end

    def persisted?; false; end

    attr_accessor :responses_attributes, :selected_response_index, :response

    def selected_response
      if selected_response_index
        responses_attributes[selected_response_index]
      else
        { id: "00000000-0000-0000-0000-000000000000", title: response || "Skipped" }
      end
    end

    private

    def assign_attributes attributes
      attributes.each do |key, value|
        send :"#{key}=", value
      end
    end

    class XMLMapper < Struct.new(:xml)
      def id
        xml.xpath("//Item")[0]["FormItemOID"]
      end

      def promis_id
        xml.xpath("//Item")[0]["ID"]
      end

      def context
        element_descriptions[0]
      end

      def title
        element_descriptions[1]
      end

      def responses
        xml.xpath("//Map").map do |xml|
          Response.new(xml: xml)
        end
      end

      def to_h
        {
          id: id,
          promis_id: promis_id,
          context: context,
          title: title,
          responses: responses,
        }
      end

      private

      def element_descriptions
        elements = xml.xpath("//Element")[0..-2] # drop last container element
        elements.map { |e| e["Description"] } # sometimes one, sometimes two
      end
    end
  end
end

