require "nokogiri"
require "active_model"

module CatEngine
  class Response < Struct.new(:id, :promis_id, :before, :title)
    include ActiveModel::Conversion
    extend ActiveModel::Naming

    def initialize attributes = nil
      assign_attributes(attributes || {})
    end

    def xml= xml
      assign_attributes XMLMapper.new(xml).to_h
    end

    def persisted?; false; end

    private

    def assign_attributes attributes
      attributes.each do |key, value|
        send :"#{key}=", value
      end
    end

    class XMLMapper < Struct.new(:xml)
      def id
        xml["ItemResponseOID"]
      end

      def promis_id
        xml["Value"]
      end

      def title
        xml["Description"]
      end

      def to_h
        {
          id: id,
          promis_id: promis_id,
          title: title,
        }
      end
    end
  end
end

